"""
This method of returning to the filtered view after submitting a form in an admin
change view is based largely upon http://djangosnippets.org/snippets/2531/

The changes:

    - Don't use a literal {} in a function/method declaration. It really is bad.
    - Use request.METHOD to determine if we want to set or use the referer.
    - Monkey patch all admin models.
    
"""

from django.contrib.admin import ModelAdmin
from django import forms

old_change_view = ModelAdmin.change_view

def change_view(self, request, object_id, extra_content=None):
    result = old_change_view(self, request, object_id, extra_content)
    
    if request.method != 'POST':
        ref = request.META.get('HTTP_REFERER', None)
        if ref:
            request.session['filtered'] = ref
    elif request.POST.has_key('_save'):
        if request.session.get('filtered', None):
            result['Location'] = request.session['filtered']
            request.session['filtered'] = None
    
    return result

ModelAdmin.change_view = change_view


from django.db.models import get_model
from django.contrib import admin

def patch_model_admin(model, patch_function):
    """
    Patch an installed ModelAdmin.
    
    patch_function must take one argument, the ModelAdmin, and
    may mutate the object in place, or return a new ModelAdmin.
    """
    if isinstance(model, basestring):
        model = get_model(*model.split('.'))
    ModelAdmin = type(admin.site._registry.get(model))
    ModelAdmin = patch_function(ModelAdmin) or ModelAdmin
    admin.site.unregister(model)
    admin.site.register(model, ModelAdmin)

def add_inlines(model, *inlines):
    """
    Add an arbitrary number of inlines to the admin for
    the provided model.
    """
    inlines = list(inlines)
    for i, inline in enumerate(inlines):
        if not issubclass(inline, admin.options.InlineModelAdmin):
            inlines[i] = type("%sInline" % inline.__name__, (admin.StackedInline,), {"model": inline})
    
    def inner(admin):
        admin.inlines = tuple(admin.inlines) + tuple(inlines)
    
    patch_model_admin(model, inner)

def add_actions(model, *actions):
    def inner(admin):
        for action in actions:
            admin.actions.append(action)
    
    patch_model_admin(model, inner)

def patch_admin(model):
    def inner(function):
        patch_model_admin(model, function)
    return inner


from distutils.core import setup
import admin_additions

setup(
    name = "django-admin-additions",
    version = admin_additions.__version__,
    description = "Admin additions.",
    long_description = open('README.rst').read(),
    url = "http://hg.schinckel.net/django-admin-additions",
    author = "Matthew Schinckel",
    author_email = "matt@schinckel.net",
    packages = [
        "admin_additions",
    ],
    classifiers = [
        'Programming Language :: Python',
        'Operating System :: OS Independent',
        'Framework :: Django',
    ],
)




from django.conf import settings
from django.contrib import admin

if hasattr(settings, 'ADMIN_ADDITIONS'):
    additions = settings.ADMIN_ADDITIONS
    
    if additions.get('RETURN_TO_FILTERED_CHANGELIST', False):
        import change_view_referer
    
    admin.ModelAdmin.save_on_top = additions.get('SAVE_ON_TOP', True)
    
    admin.options.list_select_related = additions.get('LIST_SELECT_RELATED', False)
    
    if additions.get('FULLY_DYNAMIC_FORMSETS', True):
        admin.options.InlineModelAdmin.extra = 0
    
    if additions.get('USE_LARGE_TABLE_PAGINATOR', False):
        from paginators import LargeTablePaginator
        admin.paginator = LargeTablePaginator


"""
LargeTablePaginator: from http://djangosnippets.org/snippets/2593/

Use this paginator to make admin pages load more quickly for large tables when 
using PostgreSQL. It uses the reltuples statistic instead of counting the rows 
when there is no where clause.

Original author: AlecC
"""

from django.core.paginator import Paginator

class LargeTablePaginator(Paginator):
    """
    Overrides the count method to get an estimate instead of actual count when not filtered
    """
    def _get_count(self):
        """
        Changed to use an estimate if the estimate is greater than 10,000
        Returns the total number of objects, across all pages.
        """
        if self._count is None:
            try:
                estimate = 0
                if not self.object_list.query.where:
                    try:
                        cursor = connection.cursor()
                        cursor.execute("SELECT reltuples FROM pg_class WHERE relname = %s",
                            [self.object_list.query.model._meta.db_table])
                        estimate = int(cursor.fetchone()[0])
                    except:
                        pass
                if estimate < 10000:
                    self._count = self.object_list.count()
                else:
                    self._count = estimate
            except (AttributeError, TypeError):
                # AttributeError if object_list has no count() method.
                # TypeError if object_list.count() requires arguments
                # (i.e. is of type list).
                self._count = len(self.object_list)
        return self._count
    count = property(_get_count)